@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2011-2017 by Andrew Mustun. All rights reserved.
+ * Copyright (c) 2011-2018 by Andrew Mustun. All rights reserved.
  * 
  * This file is part of the QCAD project.
  *
@@ -738,8 +738,8 @@ EAction.prototype.setCrosshairCursor = function() {
             EAction.crossCursor = new QCursor(bitmap, mask, 15, 15);
         }
         else {
-            bitmap = new QBitmap(EAction.includeBasePath + "CrosshairCursor.png", "PNG");
-            mask = new QBitmap(EAction.includeBasePath + "CrosshairCursorMask.png", "PNG");
+            bitmap = new QBitmap(EAction.includeBasePath + "/CrosshairCursor.png", "PNG");
+            mask = new QBitmap(EAction.includeBasePath + "/CrosshairCursorMask.png", "PNG");
             EAction.crossCursor = new QCursor(bitmap, mask, 15, 15);
         }
     }
