@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2011-2017 by Andrew Mustun. All rights reserved.
+ * Copyright (c) 2011-2018 by Andrew Mustun. All rights reserved.
  * 
  * This file is part of the QCAD project.
  *
@@ -35,7 +35,32 @@ ToggleLayerVisibility.prototype.beginEvent = function() {
 
     var layer = this.getCurrentLayer();
 
-    layer.setFrozen(!layer.isFrozen());
+    var showFrozen = Layer.getShowFrozen();
+    var freezeLayer = Layer.getFreezeLayer();
+
+    // separate column for frozen is shown:
+    // always toggle off:
+    if (showFrozen) {
+        layer.setOff(!layer.isOff());
+    }
+    else {
+        // freeze layers:
+        if (freezeLayer) {
+            if (!layer.isFrozen() && !layer.isOff()) {
+                layer.setFrozen(true);
+            }
+            else {
+                layer.setFrozen(false);
+                layer.setOff(false);
+            }
+        }
+
+        // never freeze layers:
+        else {
+            layer.setFrozen(false);
+            layer.setOff(!layer.isOff());
+        }
+    }
 
     var operation = new RModifyObjectOperation(layer);
     var di = this.getDocumentInterface();
