@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2011-2017 by Andrew Mustun. All rights reserved.
+ * Copyright (c) 2011-2018 by Andrew Mustun. All rights reserved.
  * 
  * Function stringToDirectDistanceEntry added 2013 by Robert S.
  *
@@ -240,6 +240,26 @@ function isBlock(obj) {
     return isOfType(obj, RBlock) || isOfType(obj, RBlockPointer);
 }
 
+/**
+ * Checks if the given object is a layout (RLayout).
+ *
+ * \return true if the given object is a layout.
+ */
+function isLayout(obj) {
+    return isOfType(obj, RLayout) || isOfType(obj, RLayoutPointer);
+}
+
+/**
+ * Checks if the given object is an RObject.
+ *
+ * \return true if the given object is an object (derived from RObject).
+ */
+function isObject(obj) {
+    return !isNull(obj) &&
+        isFunction(obj.getType) &&
+        isNumber(obj.getType());
+}
+
 /**
  * Checks if the given object is a graphics entity.
  *
@@ -248,6 +268,7 @@ function isBlock(obj) {
 function isEntity(obj) {
     return !isNull(obj) &&
         isFunction(obj.getType) && 
+        isFunction(obj.getData) &&
         isNumber(obj.getType());
 }
 
@@ -527,6 +548,15 @@ function isTextEntity(obj) {
     return isOfType(obj, RTextEntity) || isOfType(obj, RTextEntityPointer);
 }
 
+/**
+ * Checks if the given object is a simple text entity.
+ *
+ * \return true if the given object is a text entity (RTextEntity) and simple.
+ */
+function isSimpleTextEntity(obj) {
+    return isTextEntity(obj) && obj.isSimple();
+}
+
 /**
  * Checks if the given object is a block reference entity.
  *
@@ -976,15 +1006,20 @@ function getWidgetPath(widget) {
  * \param widget the parent widget
  * \param ca the initial array which holds some child widgets for
  *        recursive calls
+ * \param allowDuplicates True to collect widgets with same name under
+ * a different key ("MyField", "MyField_1", "MyField_2").
  * \return Array of child widgets.
  */
-function getWidgets(widget, ca) {
+function getWidgets(widget, ca, allowDuplicates) {
     if (isNull(widget) || isDeleted(widget)) {
         return undefined;
     }
+    if (isNull(allowDuplicates)) {
+        allowDuplicates = false;
+    }
 
     if (isNull(ca)) {
-        ca = new Object();
+        ca = {};
     }
     var children = widget.children();
     for (var i = 0; i < children.length; ++i) {
@@ -993,10 +1028,23 @@ function getWidgets(widget, ca) {
             continue;
         }
         if (!isNull(child.objectName) && child.objectName !== "") {
-            ca[child.objectName] = child;
+            if (isNull(ca[child.objectName]) || allowDuplicates===false) {
+                // new child or overwrite child:
+                ca[child.objectName] = child;
+            }
+            else {
+                // find alternative key for child:
+                for (var k=1; k<10000; k++) {
+                    if (isNull(ca[child.objectName + "_" + k])) {
+                        ca[child.objectName + "_" + k] = child;
+                        break;
+                    }
+                }
+            }
         }
+
         if (!isNull(child.children) && child.children().length !== 0) {
-            getWidgets(child, ca);
+            getWidgets(child, ca, allowDuplicates);
         }
     }
 
@@ -2079,6 +2127,7 @@ function addDirtyFlag(title) {
 }
 
 function hasPlugin(name) {
+    // TODO: refactor:
     if (name==="PROTOOLS") {
         if (new QFileInfo("scripts/Pro").exists()) {
             // PROTOOLS available not as plugins but as scripts folder:
@@ -2370,6 +2419,10 @@ function autoPath(path) {
     }
 }
 
+function exists(path) {
+    return new QFileInfo(autoPath(path)).exists();
+}
+
 function neutralPath(path) {
     if (path.startsWith(":/")) {
         return path.substring(2);
@@ -2438,6 +2491,18 @@ function writeTextFile(fileName, str) {
     file.close();
 };
 
+function getKeyboardModifiers() {
+    if (RSettings.isQt(5)) {
+        return QGuiApplication.keyboardModifiers();
+    }
+    else {
+        return QApplication.keyboardModifiers();
+    }
+};
+
+function isShiftPressed() {
+    return getKeyboardModifiers().valueOf() & Qt.ShiftModifier.valueOf();
+};
 
 // fix QPlainTextEdit API for Qt 5:
 if (!isFunction(QPlainTextEdit.prototype.toPlainText)) {
